
#include "RobboLito.h"
#include "RobboTripleBase.h"
#include <string.h>
#include "RobboTotalBase.h"

#define V VERO

RobboTripleBase *PILA_ROBBO_TRIPLE;
int NUM_ROBBO_TRIPLE = 0;
uint64 OROLOGIO;

int RobboTriple_valu (RobboTripleBase * TRIPLE, int id);

#define conto_alfiere_bianco conto_chiaro_bianco
#define conto_alfiere_nero conto_chiaro_nero

static unsigned char nMUNE[64];
static unsigned char alfMUNE[64];
static unsigned char nepedMUNE[64];
static unsigned char bipedMUNE[64];
static char ENUMn[65] =
  { A1, A2, B1, B2, A3, A4, B3, B4, C1, C2, D1, D2, C3, C4, D3, D4,
  A8, A7, B8, B7, A6, A5, B6, B5, C8, C7, D8, D7, C6, C5, D6, D5,
  H1, H2, G1, G2, H3, H4, G3, G4, F1, F2, E1, E2, F3, F4, E3, E4,
  H8, H7, G8, G7, H6, H5, G6, G5, F8, F7, E8, E7, F6, F5, E6, E5, -1
};

static char ENUM_BI_PED[49] =
  { A2, B2, C2, D2, E2, F2, G2, H2, A3, B3, C3, D3, E3, F3, G3, H3,
  A4, B4, C4, D4, E4, F4, G4, H4, A5, B5, C5, D5, E5, F5, G5, H5,
  A6, B6, C6, D6, E6, F6, G6, H6, A7, B7, C7, D7, E7, F7, G7, H7, -1
};

static char ENUM_NE_PED[49] =
  { A7, B7, C7, D7, E7, F7, G7, H7, A6, B6, C6, D6, E6, F6, G6, H6,
  A5, B5, C5, D5, E5, F5, G5, H5, A4, B4, C4, D4, E4, F4, G4, H4,
  A3, B3, C3, D3, E3, F3, G3, H3, A2, B2, C2, D2, E2, F2, G2, H2, -1
};

static char ENUM_ALF[65] =
  { A1, A3, B2, C1, A5, B4, C3, D2, E1, A7, B6, C5, D4, E3, F2, G1,
  B8, C7, D6, E5, F4, G3, H2, D8, E7, F6, G5, H4, F8, G7, H6, H8,
  H1, H3, G2, F1, H5, G4, F3, E2, D1, H7, G6, F5, E4, D3, C2, B1,
  G8, F7, E6, D5, C4, B3, A2, E8, D7, C6, B5, A4, C8, B7, A6, A8, -1
};

static int TRIPLE_PILA[16][16][16][16];


void
un_windows_triple (char *A)
{
  int s = strlen (A);
  if (A[1] == 'w')
    A[0] = A[2] - 32;
  if (A[1] == 'b')
    A[0] = A[2];
  if (A[3] == 'w')
    A[1] = A[4] - 32;
  if (A[3] == 'b')
    A[1] = A[4];
  if (A[5] == 'w')
    A[2] = A[6] - 32;
  if (A[5] == 'b')
    A[2] = A[6];
  A[s >> 1] = 0;
}

RobboTripleBase *
cog_triple (char *I)
{
  int v, p[4] = { 0, 0, 0, 0 }, i;
  if (I[0] >= '0' && I[0] <= '9')
    return PILA_ROBBO_TRIPLE + atoi (I);
  for (i = 0; i < 4 && i < strlen (I); i++)
    {
      switch (I[i])
	{
	case 'Q':
	  p[i] = conto_donna_bianca;
	  break;
	case 'q':
	  p[i] = conto_donna_nera;
	  break;
	case 'R':
	  p[i] = conto_torre_bianco;
	  break;
	case 'r':
	  p[i] = conto_torre_nero;
	  break;
	case 'B':
	  p[i] = conto_alfiere_bianco;
	  break;
	case 'b':
	  p[i] = conto_alfiere_nero;
	  break;
	case 'N':
	  p[i] = conto_cavallo_bianco;
	  break;
	case 'n':
	  p[i] = conto_cavallo_nero;
	  break;
	case 'P':
	  p[i] = conto_pedone_bianco;
	  break;
	case 'p':
	  p[i] = conto_pedone_nero;
	  break;
	}
    }
  v = TRIPLE_PILA[p[0]][p[1]][p[2]][p[3]];
  if (v == -1)
    return 0;
  return PILA_ROBBO_TRIPLE + (v % 65536);
}

void
INIZ_ROBBO_TRIPLE ()
{
  int i, j, k, l;
  for (i = A1; i <= H8; i++)
    nMUNE[(int) ENUMn[i]] = i;
  for (i = A1; i <= H8; i++)
    alfMUNE[(int) ENUM_ALF[i]] = i;
  for (i = 0; i < 48; i++)
    bipedMUNE[(int) ENUM_BI_PED[i]] = i;
  for (i = 0; i < 48; i++)
    nepedMUNE[(int) ENUM_NE_PED[i]] = i;
  for (i = 0; i < 16; i++)
    for (j = 0; j < 16; j++)
      for (k = 0; k < 16; k++)
	for (l = 0; l < 16; l++)
	  TRIPLE_PILA[i][j][k][l] = -1;
  PILA_ROBBO_TRIPLE = malloc (1024 * sizeof (RobboTripleBase));
}

void
robbo_triple_utilita (RobboTripleBase * triple, char *iNOME)
{
  int i, p[4];
  char NOME[256];
  strcpy(NOME, iNOME);
  if (NOME[0] == 'K' || NOME[0]=='k')
    un_windows_triple (NOME);
  strcpy (triple->nome, NOME);
  for (i = 0; i < 4; i++)
    {
      switch (NOME[i])
	{
	case 'Q':
	  p[i] = conto_donna_bianca;
	  break;
	case 'q':
	  p[i] = conto_donna_nera;
	  break;
	case 'R':
	  p[i] = conto_torre_bianco;
	  break;
	case 'r':
	  p[i] = conto_torre_nero;
	  break;
	case 'B':
	  p[i] = conto_alfiere_bianco;
	  break;
	case 'b':
	  p[i] = conto_alfiere_nero;
	  break;
	case 'N':
	  p[i] = conto_cavallo_bianco;
	  break;
	case 'n':
	  p[i] = conto_cavallo_nero;
	  break;
	case 'P':
	  p[i] = conto_pedone_bianco;
	  break;
	case 'p':
	  p[i] = conto_pedone_nero;
	  break;
	default:
	  p[i] = 0;
	}
      if (p[i])
	{
	  if (p[i] == conto_pedone_bianco || p[i] == conto_pedone_nero)
	    triple->m[i] = 48;
	  else
	    triple->m[i] = 64;
	}
      else
	triple->m[i] = 1;
      triple->p[i] = p[i];
    }
  if (p[0] == p[1] && p[1])
    {
      if (p[0] == p[2] && p[0] == p[1] && p[2])
	{
	  triple->m[0] =
	    (triple->m[0] * (triple->m[0] - 1) * (triple->m[0] - 2)) /
	    6;
	  triple->m[1] = triple->m[2] = 1;
	}
      else
	{
	  triple->m[0] = (triple->m[0] * (triple->m[0] - 1)) / 2;
	  triple->m[1] = 1;
	}
    }
  else if (p[1] == p[2] && p[2])
    {
      triple->m[1] = (triple->m[1] * (triple->m[1] - 1)) / 2;
      triple->m[2] = 1;
    }
  TRIPLE_PILA[p[0]][p[1]][p[2]][p[3]] =
    NUM_ROBBO_TRIPLE + (0 << 16) + (1 << 18) + (2 << 20) + (3 << 22);
  TRIPLE_PILA[p[1]][p[0]][p[2]][p[3]] =
    NUM_ROBBO_TRIPLE + (1 << 16) + (0 << 18) + (2 << 20) + (3 << 22);
  TRIPLE_PILA[p[0]][p[2]][p[1]][p[3]] =
    NUM_ROBBO_TRIPLE + (0 << 16) + (2 << 18) + (1 << 20) + (3 << 22);
  TRIPLE_PILA[p[1]][p[2]][p[0]][p[3]] =
    NUM_ROBBO_TRIPLE + (2 << 16) + (0 << 18) + (1 << 20) + (3 << 22);
  TRIPLE_PILA[p[2]][p[0]][p[1]][p[3]] =
    NUM_ROBBO_TRIPLE + (1 << 16) + (2 << 18) + (0 << 20) + (3 << 22);
  TRIPLE_PILA[p[2]][p[1]][p[0]][p[3]] =
    NUM_ROBBO_TRIPLE + (2 << 16) + (1 << 18) + (0 << 20) + (3 << 22);
}

#define reQU(x) \
  { \
    if (P[x] == conto_pedone_bianco) Q[x] = bipedMUNE[Q[x]]; \
    else if (P[x] == conto_pedone_nero) Q[x] = nepedMUNE[Q[x]]; \
    else if (P[x] == conto_alfiere_bianco || P[x] == conto_alfiere_nero) \
      Q[x] = alfMUNE[Q[x]]; \
    else Q[x] = nMUNE[Q[x]]; \
  }

static uint8 XM[16] = { 0, 1, 2, 3, 4, 4, 6, 7, 8, 9, 10, 11, 12, 12, 14, 15};

static logico
triple_opera (tipo_posizione *POSIZIONE, int *r)
{
  uint8 p[4] = { 0, 0, 0, 0 };
  int qu[4] = { 0, 0, 0, 0};
  int n = 0, v, i, x, u, r1, r2, P[4], Q[4];
  int ind, rf;
  uint64 t;
  RobboTripleBase *TRIPLE;
  t = POSIZIONE->occupato_nero_bianco ^ (bitbordo_nero_re | bitbordo_bianco_re);
  while (t)
    {
      u = BSF(t);
      p[n] = XM[POSIZIONE->qu[u]];
      qu[n++] = u;
      bitLIBERO (u, t);
    }
  if (!p[0])
    {
      *r = 0;
      return VERO;
    }
  if (POSIZIONE->bianco_en_mossa)
    {
      r1 = POSIZIONE->bianco_re_quadretto;
      r2 = POSIZIONE->nero_re_quadretto;
    }
  else
    {
      r1 = POSIZIONE->nero_re_quadretto;
      r2 = POSIZIONE->bianco_re_quadretto;
      for (i = 0; i < 4; i++)
	p[i] = col_scamb[p[i]];
    }
  v = TRIPLE_PILA[p[0]][p[1]][p[2]][p[3]];
  if ((v % 65536) == -1)
    return FALSO;
  TRIPLE = PILA_ROBBO_TRIPLE + (v % 65536);
  if (TRIPLE->w[2] == 0)
    {
      *r = 2 - TRIPLE->w[1];
      return VERO;
    }
  for (i = 0; i < 4; i++)
    {
      x = (v >> (16 + 2 * i)) & 3;
      Q[i] = qu[x];
      P[i] = p[x];
    }
  if (bitbordo_bianco_pedone | bitbordo_nero_pedone)
    {
      int w = (2 * (COLONNA (r2) >= CE)) + (!POSIZIONE->bianco_en_mossa);
      for (i = 0; i < 4; i++)
	LANCpez (w, Q[i]);
      LANCpez (w, r1);
      LANCpez (w, r2);
      rf = RE_pedone[64 * r1 + r2];
    }
  else
    {
      uint8 w = oLANC[r1][r2];
      for (i = 0; i < 4; i++)
	LANCnor (w, Q[i]);
      LANCnor (w, r1);
      LANCnor (w, r2);
      rf = RE_normale[64 * r1 + r2];
    }
  if (rf == -1)
    return FALSO;

  for (i = 0; i < 4; i++)
    reQU (i);
  for (i = 0; i < 4; i++)
    if (!P[i])
      Q[i] = 0;
  if (P[0] == P[2] && P[2] && P[1] == P[2])
    {
      if (Q[1] < Q[2])
	SCAMB (Q[1], Q[2]);
      if (Q[0] < Q[1])
	SCAMB (Q[0], Q[1]);
      if (Q[1] < Q[2])
	SCAMB (Q[1], Q[2]);
      Q[0] =
	(Q[0] * (Q[0] - 1) * (Q[0] - 2)) / 6 + (Q[1] * (Q[1] + 1)) / 2 -
	Q[1] + Q[2];
      Q[1] = Q[2] = 0;
    }
  else if (P[0] == P[1] && P[1])
    {
      if (Q[0] < Q[1])
	SCAMB (Q[0], Q[1]);
      Q[0] = (Q[0] * (Q[0] + 1)) / 2 + Q[1] - Q[0];
      Q[1] = 0;
    }
  else if (P[1] == P[2] && P[2])
    {
      if (Q[1] < Q[2])
	SCAMB (Q[1], Q[2]);
      Q[1] = (Q[1] * (Q[1] + 1)) / 2 + Q[2] - Q[1];
      Q[2] = 0;
    }
  ind =
    ((((rf * TRIPLE->m[0]) + Q[0]) * TRIPLE->m[1] + Q[1]) * TRIPLE->m[2] +
     Q[2]) * TRIPLE->m[3] + Q[3];
  *r = 2 - RobboTriple_valu (TRIPLE, ind);
  return VERO;
}

#define vaVIT 1
#define vaPAT 0
#define vaPER -1

#define SCACCO \
  (POSIZIONE->bianco_en_mossa ? \
   (bitbordo_bianco_re&DINAMICO->attacco_nero) : \
   (bitbordo_nero_re&DINAMICO->attacco_bianco) )
#define ILLEGALE \
  (!POSIZIONE->bianco_en_mossa ? \
   (bitbordo_bianco_re&DINAMICO->attacco_nero) : \
   (bitbordo_nero_re&DINAMICO->attacco_bianco))

logico
TRIPLE_VALU (tipo_posizione *POSIZIONE, int *valu)
{
  int r = vaPER, b = vaPER, i, mossa;
  if (DINAMICO->arrocco || DINAMICO->en_passant) return FALSO;
  if (ILLEGALE)
    return FALSO;
#ifdef ROBBO_SCACCO
  if (POPCNT (POSIZIONE->occupato_nero_bianco) >= 5 && (u = SCACCO))
    {
      lista = evasione_mossa (LISTA, 0xffffffffffffffff);
      if (lista == LISTA)
	{
	  *valu = vaPER;
	  return VERO;
	}
      for (i = 0; i < lista - LISTA; i++)
	{
	  mossa = LISTA[i].mossa;
	  fare (mossa);
	  mobilita ();
	  if (ILLEGALE)
	    {
	      annullare (mossa);
	      continue;
	    }
	  if (!triple_RICER (&r))
	    {
	      annullare (mossa);
	      return FALSO;
	    }
	  r = -r;
	  annullare (mossa);
	  if (r == vaVIT)
	    {
	      *valu = vaVIT;
	      return VERO;
	    }
	  if (r == vaPAT)
	    b = vaPAT;
	}
      *valu = b;
      return VERO;
    }
#endif
  tipo_lista_mossa LISTA[256], *lista;
#if ROBBO_TRIPLE_BUILD		/* pigro */
  lista = cattura_gen_total (POSIZIONE, LISTA);
#else
  lista = cattura_mossa (POSIZIONE, LISTA, 0xffffffffffffffff);
#endif
  for (i = 0; i < lista - LISTA; i++)
    {
      mossa = LISTA[i].mossa;
      fare (POSIZIONE, mossa);
      mobilita (POSIZIONE);
      if (!ILLEGALE)
	{
	  if (!TRIPLE_VALU (POSIZIONE, &r))
	    {
	      annullare (POSIZIONE, mossa);
	      return FALSO;
	    }
	  r = -r;
	}
      annullare (POSIZIONE, mossa);
      if (r == vaVIT)
	{
	  *valu = r;
	  return VERO;
	}
      if (r == vaPAT)
	b = vaPAT;
    }
  if (!triple_opera (POSIZIONE, &r))
    return FALSO;
  if (b > r)
    *valu = b;
  else
    *valu = r;
  return VERO;
}

static const uint8 NX[4][2] = { {0, 0}, {2, 3}, {1, 3}, {1, 2} };

static int
ott_data (uint8 * E, int ba, uint8 * AA, uint8 * A, int PR, int sb)
{
  int n, b, i = 0, r = A[sb];
  uint8 ch, tr;
  while (ba >= 0 && i < PR)
    {
      ch = E[i++];
      tr = (ch & 64) >> 6;
      n = ch & 31;
      b = 0;
      if (ch & 32)
	{
	  if (!ba)
	    return AA[NX[r][tr]];
	  else
	    ba--;
	}
      else
	r = NX[r][tr];
      while (ch & 128)
	{
	  b++;
	  ch = E[i++];
	  n += ((ch & 127) << (7 * b - 2));
	}
      ba -= n;
    }
  return AA[r];
}

int
RobboTriple_valu (RobboTripleBase * TRIPLE, int i)
{
  int w = 0, l, u;
  if (TRIPLE->w[2] == 0)
    return (TRIPLE->w[1]);
  l = TRIPLE->bs;
  if (l == -1)
    w = 0;
  else
    {
      w = (2 << l) - 1;
      while (l >= 0)
	{
	  u = TRIPLE->ind[w] >> 2;
	  w += ((i > u) - (i < u)) << l;
	  l--;
	}
      while (i >= (TRIPLE->ind[w] >> 2))
	w++;
      w--;
    }
  return ott_data ((TRIPLE->data) + (TRIPLE->prop_ind * w),
		   i - (TRIPLE->ind[w] >> 2), TRIPLE->w, TRIPLE->wi,
		   TRIPLE->prop_ind, TRIPLE->ind[w] & 3);
}
