
#include "RobboLito.h"
#include "RobboTotalBase.h"

static void
iniz_RE_normale ()
{
  int a, b, s = 0;
  for (a = A1; a <= H8; a++)
    for (b = A1; b <= H8; b++)
      {
	if (a == b || attaco_re[a] & quadretto_fisso[b])
	  {
	    RE_normale[64 * b + a] = -1;
	    continue;
	  }
	if (!(TRIANGOLO & quadretto_fisso[a]))
	  {
	    RE_normale[64 * b + a] = -1;
	    continue;
	  }
	if (MARGINE & quadretto_fisso[a] && !(TRI_MAR & quadretto_fisso[b]))
	  {
	    RE_normale[64 * b + a] = -1;
	    continue;
	  }
	RE_normale[64 * b + a] = s++;
      }
}

static void
iniz_RE_pedone ()
{
  int a, b, s = 0;
  for (a = A1; a <= H8; a++)
    for (b = A1; b <= H8; b++)
      {
	if (a == b || attaco_re[a] & quadretto_fisso[b])
	  {
	    RE_pedone[64 * b + a] = -1;
	    continue;
	  }
	if (COLONNA (a) >= CE)
	  {
	    RE_pedone[64 * b + a] = -1;
	    continue;
	  }
	RE_pedone[64 * b + a] = s++;
      }
}

static void
iniz_inverso ()
{
  int br, nr;
  for (br = A1; br <= H8; br++)
    for (nr = A1; nr <= H8; nr++)
      {
	if (RE_normale[64 * br + nr] != -1)
	  inverso_normale_re[RE_normale[64 * br + nr]] = (br << 6) | nr;
	if (RE_pedone[64 * br + nr] != -1)
	  inverso_pedone_re[RE_pedone[64 * br + nr]] = (br << 6) | nr;
      }
}

static void
iniz_doppio2 ()
{
  int a, b, s = 0;
  for (a = A1; a <= H8; a++)
    for (b = A1; b < a; b++)
      quad_due_normale[s++] = a | (b << 6);
  s = 0;
  for (a = A2; a <= H7; a++)
    for (b = A2; b < a; b++)
      quad_due_pedone[s++] = a | (b << 6);
}

static void
iniz_triplo3 ()
{
  int a, b, c, s = 0;
  for (a = A1; a <= H8; a++)
    for (b = A1; b < a; b++)
      for (c = A1; c < b; c++)
	quad_tre_normale[s++] = a | (b << 6) | (c << 12);
  s = 0;
  for (a = A2; a <= H7; a++)
    for (b = A2; b < a; b++)
      for (c = A2; c < b; c++)
	quad_tre_pedone[s++] = a | (b << 6) | (c << 12);
}

static void
iniz_quadruplo4 ()
{
  int a, b, c, d, s = 0;
  for (a = A1; a <= H8; a++)
    for (b = A1; b < a; b++)
      for (c = A1; c < b; c++)
	for (d = A1; d < c; d++)
	  quad_quat_normale[s++] = a | (b << 6) | (c << 12) | (d << 18);
  s = 0;
  for (a = A2; a <= H7; a++)
    for (b = A2; b < a; b++)
      for (c = A2; c < b; c++)
	for (d = A2; d < c; d++)
	  quad_quat_pedone[s++] = a | (b << 6) | (c << 12) | (d << 18);
}

static void
iniz_lanc ()
{
  int br, nr, r1, r2, u, qu;
  for (br = A1; br <= H8; br++)
    for (nr = A1; nr <= H8; nr++)
      {
	r1 = nr;
	r2 = br;
	u = 0;
	if (COLONNA (r1) >= CE)
	  {
	    u = 4;
	    LANCnor (4, r1);
	    LANCnor (4, r2);
	  }
	if (TRAVERSA (r1) >= T5)
	  {
	    u |= 2;
	    LANCnor (2, r1);
	    LANCnor (2, r2);
	  }
	if (TRAVERSA (r1) == COLONNA (r1) && TRAVERSA (r2) > COLONNA (r2))
	  u |= 1;
	else if (TRAVERSA (r1) > COLONNA (r1))
	  u |= 1;
	oLANC[br][nr] = u;
      }
  for (qu = A1; qu <= H8; qu++)
    {
      lanc_pedone[0][qu] = qu;
      lanc_pedone[1][qu] = lanc_vert[qu];
      lanc_pedone[2][qu] = lanc_oriz[qu];
      lanc_pedone[3][qu] = (CH - COLONNA (qu)) + 8 * (T8 - TRAVERSA (qu));
    }
}

static void
iniz_lanc_normale ()
{
  int i, qu, w;
  for (i = 0; i < 8; i++)
    for (qu = A1; qu <= H8; qu++)
      {
	w = qu;
	if (i & 4)
	  w = lanc_oriz[w];
	if (i & 2)
	  w = lanc_vert[w];
	if (i & 1)
	  w = lanc_diag[w];
	lanc_norm[i][qu] = w;
      }
}

static void
iniz_doppio ()
{
  int i, j;
  for (i = A1; i <= H8; i++)
    for (j = A1; j <= H8; j++)
      {
	STESSO2[i][j] =
	  (i > j) ? (i * (i + 1)) / 2 + j - i : (j * (j + 1)) / 2 + i - j;
      }
}

static void
iniz_triplo ()
{
  int i, j, k, mas, mez, min;
  for (i = A1; i <= H8; i++)
    for (j = A1; j <= H8; j++)
      for (k = A1; k <= H8; k++)
	{
	  mas = MASSIMO (MASSIMO (i, j), k);
	  min = MINIMO (MINIMO (i, j), k);
	  if (i == mas)
	    mez = MASSIMO (j, k);
	  if (j == mas)
	    mez = MASSIMO (i, k);
	  if (k == mas)
	    mez = MASSIMO (i, j);
	  STESSO3[i][j][k] =
	    (mas * (mas - 1) * (mas - 2)) / 6 +
	    (mez * (mez + 1)) / 2 - mez + min;
	}
}

static void
iniz_quadruplo ()
{
  int i, j, k, l, mas, mez2, mez3, min;
  for (i = A1; i <= H8; i++)
    for (j = A1; j <= H8; j++)
      for (k = A1; k <= H8; k++)
	for (l = A1; l <= H8; l++)
	{
	  mas = MASSIMO (MASSIMO (i, j), MASSIMO (k, l));
	  min = MINIMO (MINIMO (i, j), MINIMO (k, l));
	  if (i == mas)
	    mez2 = MASSIMO (MASSIMO (j, k), l);
	  if (j == mas)
	    mez2 = MASSIMO (MASSIMO (i, k), l);
	  if (k == mas)
	    mez2 = MASSIMO (MASSIMO (i, j), l);
	  if (l == mas)
	    mez2 = MASSIMO (MASSIMO (i, j), k);
	  if (i == min)
	    mez3 = MINIMO (MINIMO (j, k), l);
	  if (j == min)
	    mez3 = MINIMO (MINIMO (i, k), l);
	  if (k == min)
	    mez3 = MINIMO (MINIMO (i, j), l);
	  if (l == min)
	    mez3 = MINIMO (MINIMO (i, j), k);
	  STESSO4[i][j][k][l] =
	    (mas * (mas - 1) * (mas - 2) * (mas - 3)) / 24 +
	    (mez2 * (mez2 - 1) * (mez2 - 2)) / 6 +
	    (mez3 * (mez3 - 1)) / 2 + min;
	}
}

static void
pila_lanc ()
{
  int pezzo, qu;
  for (pezzo = 0; pezzo < 16; pezzo++)
    col_scamb[pezzo] = pezzo ^ 8;
  col_scamb[0] = 0;
  for (qu = A1; qu <= H8; qu++)
    {
      lanc_oriz[qu] = (CH - COLONNA (qu)) + 8 * TRAVERSA (qu);
      lanc_vert[qu] = COLONNA (qu) + 8 * (T8 - TRAVERSA (qu));
      lanc_diag[qu] = 8 * COLONNA (qu) + TRAVERSA (qu);
    }
}

void
iniz_robbo_utilita (logico Build)
{
  pila_lanc ();
  strcpy (ROBBO_TOTAL_DIR, "RobboTotalBase");
  strcpy (COMPRESSA_PREFISSO, "iCOMP.");
  iniz_RE_normale ();
  iniz_RE_pedone ();
  iniz_inverso ();
  iniz_doppio2 ();
  iniz_triplo3 ();
  if (Build) iniz_quadruplo4 ();
  iniz_lanc_normale ();
  iniz_lanc ();
  iniz_doppio ();
  iniz_triplo ();
  if (Build) iniz_quadruplo ();
}

static int
canonico_windows (char *AA, logico * simm)
{
  char SORTA[16] = "KQRBNP";
  char A[32];
  int j, n, n2, i, SO[32];
  strcpy(A, AA);
  A[16] = 0;
  for (i = 0; i < 16; i++)
    if (A[i] == '0')
      A[i] = 0;
  for (n = 0; n < 16; n++)
    if (!A[n]) break;
  for (i = n; i < 16; i++)
    A[i] = 0;
  for (i = 0; i < 16; i++)
    if (A[i] >= 'a')
      A[i] -= 32;
  for (i = 0; i < 16; i++)
    if (A[i] == 0)
      for (j = i; j < 4; j++)
	A[j] = 0;
  for (i = 0; i < 16; i++)
    if (!index (SORTA, A[i]))
      A[i] = 0;
  for (n = 1; A[n] != 'K'; n++);
  for (i = 0; i < n; i++)
    for (j = 0; j < strlen (SORTA); j++)
      if (SORTA[j] == A[i])
	SO[i] = j;
  for (i = n - 1; i >= 1; i--)
    for (j = i + 1; j < n && SO[j - 1] > SO[j]; j++)
      {
	SCAMB (A[j - 1], A[j]);
	SCAMB (SO[j - 1], SO[j]);
      }
  for (n2 = n + 1; A[n2] ; n2++);
  for (i = n; i < n2; i++)
    for (j = 0; j < strlen (SORTA); j++)
      if (SORTA[j] == A[i])
	SO[i] = j;
  for (i = n + 1; i < n2; i++)
    for (j = 0; j < strlen (SORTA); j++)
      if (SORTA[j] == A[i])
	SO[i] = j;
  for (i = n2 - 1; i >= n + 1; i--)
    for (j = i + 1; j < n2 && SO[j - 1] > SO[j]; j++)
      {
	SCAMB (A[j - 1], A[j]);
	SCAMB (SO[j - 1], SO[j]);
      }

  if (n2 == 2)
    *simm = VERO;
  if (n2 == 4 && SO[1] == SO[3])
    *simm = VERO;
  if (n2 == 6 && SO[1] == SO[4] && SO[2] == SO[5])
    *simm = VERO;

  return n;
}

int
nome_canonico (char *A, logico * simm)
{
  char SORTA[16] = "QRBNPqrbnp";
  int j, n, i, SO[MASSIMO_NUMERO];
  if (A[0] == 'K' || A[0]=='k')
    return canonico_windows (A, simm);
  A[MASSIMO_NUMERO] = 0;
  for (i = 0; i < MASSIMO_NUMERO; i++)
    if (A[i] == '0')
      A[i] = 0;
  for (i = 0; i < MASSIMO_NUMERO; i++)
    if (A[i] == 0)
      for (j = i; j < 4; j++)
	A[j] = 0;
  for (i = 0; i < MASSIMO_NUMERO; i++)
    if (!index (SORTA, A[i]))
      A[i] = 0;
  for (n = 0; A[n]; n++);
  for (i = 0; i < n; i++)
    for (j = 0; j < strlen (SORTA); j++)
      if (SORTA[j] == A[i])
	SO[i] = j;
  for (i = n - 1; i >= 0; i--)
    for (j = i + 1; j < n && SO[j - 1] > SO[j]; j++)
      {
	SCAMB (A[j - 1], A[j]);
	SCAMB (SO[j - 1], SO[j]);
      }
  if (n == 0)
    *simm = VERO;
  if (n == 2 && (SO[0] + 5) == SO[1])
    *simm = VERO;
  if (n == 4 && (SO[0] + 5) == SO[2] && (SO[1] + 5) == SO[3])
    *simm = VERO;
  return n;
}

int
RobboTotal_scop (char *NOME)
{
  int i, p[4] = { 0, 0, 0, 0 }, v;
  for (i = 0; i < 4 && NOME[i]; i++)
    {
      switch (NOME[i])
	{
	case 'Q':
	  p[i] = conto_donna_bianca;
	  break;
	case 'q':
	  p[i] = conto_donna_nera;
	  break;
	case 'R':
	  p[i] = conto_torre_bianco;
	  break;
	case 'r':
	  p[i] = conto_torre_nero;
	  break;
	case 'B':
	  p[i] = conto_alfiere_bianco;
	  break;
	case 'b':
	  p[i] = conto_alfiere_nero;
	  break;
	case 'N':
	  p[i] = conto_cavallo_bianco;
	  break;
	case 'n':
	  p[i] = conto_cavallo_nero;
	  break;
	case 'P':
	  p[i] = conto_pedone_bianco;
	  break;
	case 'p':
	  p[i] = conto_pedone_nero;
	  break;
	default:
	  p[i] = 0;
	}
    }
  v = RTB_PILA[p[0]][p[1]][p[2]][p[3]];
  if (v != -1)
    v &= 65535;
  return v;
}

void
RobboTotal_sott (char *NOME)
{
  int i, p[4] = { 0, 0, 0, 0 }, v;
  int i0, i1, i2, i3;
  for (i = 0; i < 4 && NOME[i]; i++)
    {
      switch (NOME[i])
	{
	case 'Q':
	  p[i] = conto_donna_bianca;
	  break;
	case 'q':
	  p[i] = conto_donna_nera;
	  break;
	case 'R':
	  p[i] = conto_torre_bianco;
	  break;
	case 'r':
	  p[i] = conto_torre_nero;
	  break;
	case 'B':
	  p[i] = conto_alfiere_bianco;
	  break;
	case 'b':
	  p[i] = conto_alfiere_nero;
	  break;
	case 'N':
	  p[i] = conto_cavallo_bianco;
	  break;
	case 'n':
	  p[i] = conto_cavallo_nero;
	  break;
	case 'P':
	  p[i] = conto_pedone_bianco;
	  break;
	case 'p':
	  p[i] = conto_pedone_nero;
	  break;
	default:
	  p[i] = 0;
	}
    }
  v = RTB_PILA[p[0]][p[1]][p[2]][p[3]];
  if (v != -1)
    v &= 65535;
  if (v != -1 && (PILA_ROBBO_TOTAL + v)->Fdata)
    fclose ((PILA_ROBBO_TOTAL + v)->Fdata);
#define RTBx(a,b,c,d) RTB_PILA[p[a]][p[b]][p[c]][p[d]]=-1;
  for (i0 = 0; i0 < 4; i0++)
    for (i1 = 0; i1 < 4; i1++)
      {
	if (i0 == i1)
	  continue;
	for (i2 = 0; i2 < 4; i2++)
	  {
	    if (i2 == i1 || i2 == i0)
	      continue;
	    for (i3 = 0; i3 < 4; i3++)
	      {
		if (i3 == i2 || i3 == i1 || i3 == i0)
		  continue;
		RTBx (i0, i1, i2, i3);
	      }
	  }
      }
}

#define VERBOSO FALSO

void
un_windows (char *NOME)
{
  int c = 1, w = 0;
  while (NOME[c] != 'k' && NOME[c] != 'K')
    {
      NOME[w] = NOME[c++];
      if (NOME[w] > 96) NOME[w] -= 32;
      w++;
    }
  c++;
  while (NOME[c])
    {
      NOME[w] = NOME[c++];
      if (NOME[w] < 96) NOME[w] += 32;
      w++;
    }
  for ( ; w <= 4 ; w++)
    NOME[w] = 0;
}

void
RobboTotal_add (RobboTotalBase * TotalBase, char *NOME)
{
  int i, j, p[4];
  char NOME2[16];
  logico PEDONE_PIETRA = FALSO;
  if (NOME[0] == 'K' || NOME[0]=='k')
    un_windows (NOME);
  if (NOME[0] == '.')
    return;
  if (RobboTotal_scop (NOME) != -1)
    return;
  for (i = 0; i < 4; i++)
    if (NOME[i] == '0')
      NOME[i] = 0;
  for (i = 3; NOME[i - 2]; i++);
#ifdef NOME_WINDOWS
  NOME2[0] = 'K';
  j = 0;
  int c = 1;
  while (NOME[j] >= 'A' && NOME[j] <= 'Z')
    NOME2[c++] = NOME[j++];
  NOME2[c++] = 'K';
  while (NOME[j])
    NOME2[c++] = NOME[j++] - 32;
  NOME2[c]=0;
#else
  strcpy(NOME2, NOME);
#endif
  if (VERBOSO)
    printf ("Adding RobboTotalBase #%d %s\n", NUM_ROBBO_TOTAL, NOME2);
  sprintf (TotalBase->DIR_NOME, "%s/%d/", ROBBO_TOTAL_DIR, i);
  strcpy (TotalBase->string, NOME2);
  for (i = 0; i < 4; i++)
    {
      switch (NOME[i])
	{
	case 'Q':
	  p[i] = conto_donna_bianca;
	  break;
	case 'q':
	  p[i] = conto_donna_nera;
	  break;
	case 'R':
	  p[i] = conto_torre_bianco;
	  break;
	case 'r':
	  p[i] = conto_torre_nero;
	  break;
	case 'B':
	  p[i] = conto_alfiere_bianco;
	  break;
	case 'b':
	  p[i] = conto_alfiere_nero;
	  break;
	case 'N':
	  p[i] = conto_cavallo_bianco;
	  break;
	case 'n':
	  p[i] = conto_cavallo_nero;
	  break;
	case 'P':
	  p[i] = conto_pedone_bianco;
	  break;
	case 'p':
	  p[i] = conto_pedone_nero;
	  break;
	default:
	  p[i] = 0;
	}
      if (p[i])
	{
	  if (p[i] == conto_pedone_bianco || p[i] == conto_pedone_nero)
	    TotalBase->m[i] = 48;
	  else
	    TotalBase->m[i] = 64;
	}
      else
	TotalBase->m[i] = 1;
    }
  if (p[0] && p[0] == p[3])
    {
      TotalBase->m[0] =
	(TotalBase->m[0] * (TotalBase->m[0] - 1) *
	 (TotalBase->m[0] - 2) * (TotalBase->m[0] - 3)) / 24;
      TotalBase->m[1] = TotalBase->m[2] = TotalBase->m[3] = 1;
    }
  else if (p[0] && p[0] == p[2])
    {
      TotalBase->m[0] =
	(TotalBase->m[0] * (TotalBase->m[0] - 1) * (TotalBase->m[0] - 2)) / 6;
      TotalBase->m[1] = TotalBase->m[2] = 1;
    }
  else if (p[1] && p[1] == p[3])
    {
      TotalBase->m[1] =
	(TotalBase->m[1] * (TotalBase->m[1] - 1) * (TotalBase->m[1] - 2)) / 6;
      TotalBase->m[2] = TotalBase->m[3] = 1;
    }
  else
    {
      if (p[0] && p[0] == p[1])
	{
	  TotalBase->m[0] = (TotalBase->m[0] * (TotalBase->m[0] - 1)) / 2;
	  TotalBase->m[1] = 1;
	}
      if (p[1] && p[1] == p[2])
	{
	  TotalBase->m[1] = (TotalBase->m[1] * (TotalBase->m[1] - 1)) / 2;
	  TotalBase->m[2] = 1;
	}
      if (p[2] && p[2] == p[3])
	{
	  TotalBase->m[2] = (TotalBase->m[2] * (TotalBase->m[2] - 1)) / 2;
	  TotalBase->m[3] = 1;
	}
    }
  for (i = 0; i < 4; i++)
    {
      TotalBase->p[i] = p[i];
      if (p[i] == conto_pedone_bianco || p[i] == conto_pedone_nero)
	PEDONE_PIETRA = VERO;
    }
  TotalBase->PEDONE = PEDONE_PIETRA;
  TotalBase->pro = 1;
  for (i = 0; i < 4; i++)
    TotalBase->pro *= TotalBase->m[i];
  TotalBase->num = NUM_ROBBO_TOTAL;
  TotalBase->pezzo_molt = TotalBase->pro;
  if (!PEDONE_PIETRA)
    TotalBase->pro *= 462;
  else
    TotalBase->pro *= 1806;
  RTB_PILA[p[0]][p[1]][p[2]][p[3]] =
    NUM_ROBBO_TOTAL + (0 << 16) + (1 << 18) + (2 << 20) + (3 << 22);
  RTB_PILA[p[1]][p[0]][p[2]][p[3]] =
    NUM_ROBBO_TOTAL + (1 << 16) + (0 << 18) + (2 << 20) + (3 << 22);
  RTB_PILA[p[0]][p[2]][p[1]][p[3]] =
    NUM_ROBBO_TOTAL + (0 << 16) + (2 << 18) + (1 << 20) + (3 << 22);
  RTB_PILA[p[1]][p[2]][p[0]][p[3]] =
    NUM_ROBBO_TOTAL + (2 << 16) + (0 << 18) + (1 << 20) + (3 << 22);
  RTB_PILA[p[2]][p[0]][p[1]][p[3]] =
    NUM_ROBBO_TOTAL + (1 << 16) + (2 << 18) + (0 << 20) + (3 << 22);
  RTB_PILA[p[2]][p[1]][p[0]][p[3]] =
    NUM_ROBBO_TOTAL + (2 << 16) + (1 << 18) + (0 << 20) + (3 << 22);

  RTB_PILA[p[0]][p[1]][p[3]][p[2]] =
    NUM_ROBBO_TOTAL + (0 << 16) + (1 << 18) + (3 << 20) + (2 << 22);
  RTB_PILA[p[0]][p[2]][p[3]][p[1]] =
    NUM_ROBBO_TOTAL + (0 << 16) + (3 << 18) + (1 << 20) + (2 << 22);
  RTB_PILA[p[1]][p[0]][p[3]][p[2]] =
    NUM_ROBBO_TOTAL + (1 << 16) + (0 << 18) + (3 << 20) + (2 << 22);
  RTB_PILA[p[1]][p[2]][p[3]][p[0]] =
    NUM_ROBBO_TOTAL + (3 << 16) + (0 << 18) + (1 << 20) + (2 << 22);
  RTB_PILA[p[2]][p[0]][p[3]][p[1]] =
    NUM_ROBBO_TOTAL + (1 << 16) + (3 << 18) + (0 << 20) + (2 << 22);
  RTB_PILA[p[2]][p[1]][p[3]][p[0]] =
    NUM_ROBBO_TOTAL + (3 << 16) + (1 << 18) + (0 << 20) + (2 << 22);

  RTB_PILA[p[0]][p[3]][p[1]][p[2]] =
    NUM_ROBBO_TOTAL + (0 << 16) + (2 << 18) + (3 << 20) + (1 << 22);
  RTB_PILA[p[0]][p[3]][p[2]][p[1]] =
    NUM_ROBBO_TOTAL + (0 << 16) + (3 << 18) + (2 << 20) + (1 << 22);
  RTB_PILA[p[1]][p[3]][p[0]][p[2]] =
    NUM_ROBBO_TOTAL + (2 << 16) + (0 << 18) + (3 << 20) + (1 << 22);
  RTB_PILA[p[1]][p[3]][p[2]][p[0]] =
    NUM_ROBBO_TOTAL + (3 << 16) + (0 << 18) + (2 << 20) + (1 << 22);
  RTB_PILA[p[2]][p[3]][p[0]][p[1]] =
    NUM_ROBBO_TOTAL + (2 << 16) + (3 << 18) + (0 << 20) + (1 << 22);
  RTB_PILA[p[2]][p[3]][p[1]][p[0]] =
    NUM_ROBBO_TOTAL + (3 << 16) + (2 << 18) + (0 << 20) + (1 << 22);

  RTB_PILA[p[3]][p[0]][p[1]][p[2]] =
    NUM_ROBBO_TOTAL + (1 << 16) + (2 << 18) + (3 << 20) + (0 << 22);
  RTB_PILA[p[3]][p[0]][p[2]][p[1]] =
    NUM_ROBBO_TOTAL + (1 << 16) + (3 << 18) + (2 << 20) + (0 << 22);
  RTB_PILA[p[3]][p[1]][p[0]][p[2]] =
    NUM_ROBBO_TOTAL + (2 << 16) + (1 << 18) + (3 << 20) + (0 << 22);
  RTB_PILA[p[3]][p[1]][p[2]][p[0]] =
    NUM_ROBBO_TOTAL + (3 << 16) + (1 << 18) + (2 << 20) + (0 << 22);
  RTB_PILA[p[3]][p[2]][p[0]][p[1]] =
    NUM_ROBBO_TOTAL + (2 << 16) + (3 << 18) + (1 << 20) + (0 << 22);
  RTB_PILA[p[3]][p[2]][p[1]][p[0]] =
    NUM_ROBBO_TOTAL + (3 << 16) + (2 << 18) + (1 << 20) + (0 << 22);
  NUM_ROBBO_TOTAL++;
}

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

logico
RobboTotal_registro (char *nome)
{
  int i, j;
  RobboTotalBase *TotalBase;
  char NOME[64], NOME_FILE[1024];
  FILE *F;
  uint8 A[4];
  uint32 n = 0;
  int num_indici = 0;
  int e = 0;
  strcpy (NOME, nome);
  if (NOME[0] == 'K' || NOME[0] == 'k')
    un_windows (NOME);
  for (i = 0; i < 4; i++)
    if (NOME[i] == '0')
      NOME[i] = 0;
  for (i = 0; i < 4; i++)
    if (NOME[i] == 0)
      for (j = i; j < 4; j++)
	NOME[j] = 0;
  NOME[4] = 0;
  for (i = 3; NOME[i - 2]; i++);

  if (RobboTotal_scop (NOME) != -1)
    return FALSO;
  TotalBase = PILA_ROBBO_TOTAL + NUM_ROBBO_TOTAL;
  TotalBase->TIPO = DISCO_COMPRESSO;
  RobboTotal_add (TotalBase, NOME);
  sprintf (NOME_FILE, "%s%s%s", TotalBase->DIR_NOME, COMPRESSA_PREFISSO,
	   TotalBase->string);
  F = fopen (NOME_FILE, "r");
  if (!F)
    return FALSO;
  fread (A, 1, 4, F);
  if (A[0] != 0xca || A[1] != 0x7f || A[2] != 0x0b || A[3] != 0xaa)
    {
      fclose (F);
      return FALSO;
    }
  fread (A, 1, 4, F);
  for (i = 0; i < 4; i++)
    n |= A[i] << (8 * i);
  TotalBase->BLOCCO_PRO = n & ((1 << 28) - 1);
  TotalBase->uBWT = n >> 28;
  fread (A, 1, 4, F);
  fread (A, 1, 4, F);
  for (i = 0; i < 4; i++)
    num_indici |= A[i] << (8 * i);
  TotalBase->indici = malloc (4 * num_indici);
  fread (TotalBase->indici, 4, num_indici, F);
  fclose (F);
  for (i = 0; i < 8; i++)
    TotalBase->supp_indice[i] = 0xffffffff;
  for (i = 1; i <= num_indici - 1; i++)
    if (TotalBase->indici[i] < TotalBase->indici[i - 1])
      TotalBase->supp_indice[e++] = i;
  sprintf (NOME_FILE, "%s%s%s", TotalBase->DIR_NOME, COMPRESSA_PREFISSO + 1,
	   TotalBase->string);
  TotalBase->Fdata = NULL;
  struct stat buf[1];
  if (stat (NOME_FILE, buf) == -1)
    {
      free (TotalBase->indici);
      return FALSO;
    }
  return VERO;
}

#include <fcntl.h>
#include <dirent.h>
#include <string.h>

#define WINDOWS_PREFISSO "icomp."
static void
add_nome_total (char *NOME)
{
  int n = strlen (COMPRESSA_PREFISSO);
  if (memcmp (NOME, COMPRESSA_PREFISSO, n) && memcmp (NOME, WINDOWS_PREFISSO, n))
    return;
  RobboTotal_registro (NOME + n);
}

void
robbo_total_directory (char *A)
{
  DIR *dir;
  struct dirent *DE;
  printf ("Reading directory %s\n", A);
  dir = opendir (A);
  if (!dir)
    {
      printf ("Directory %s not found\n", A);
      return;
    }
  while ((DE = readdir (dir)))
    add_nome_total (DE->d_name);
  closedir (dir);
}
